#!/usr/bin/wish

#########################################################
#                                                       #
#              GenomePixelizer 2D Plot                  #
#              -----------------------                  #
# Author: Alexander Kozik & Brian Chan                  #
# Date:	  July 23 2003                                  #
#                                                       #
#########################################################

#########################################################
# global setting
#########################################################

global coord_filename
global matrix_filename
global annotation_file
global var_X_dir
global var_X_start
global var_X_end
global var_Y_dir
global var_Y_start
global var_Y_end
global var_cutoff
global var_canvas_size_X
global var_canvas_size_Y
global chromosome_length

global var_sizeOf_gene
global var_gene_fill_color
global var_distance_from_line
global var_distance_from_ruler
global var_gradient_plot
global var_post_script_file
global font_color_canvas
global var_interval_num

global tag_abc
global color_scheme
global message_text

set font_color_canvas "black"
set var_gradient_plot "Yes"
set color_scheme "WHITE"
set message_text "Message Box"

set coord_filename "Matrix/ath_ncbi.coords"
set matrix_filename "Matrix/ath_ncbi.matrix"
set annotation_file "Matrix/ath_ncbi.annotation"
set var_post_script_file "genopix_2D.ps"

set var_X_dir 1
set var_X_start 0
set var_X_end "END"
set var_Y_dir 1
set var_Y_start 0
set var_Y_end "END"
set var_cutoff 0.4
set var_canvas_size_X_userInput 1000.00
set var_canvas_size_Y_userInput 1000.00
set var_sizeOf_gene 3
set var_margin_X 40
set var_margin_Y 50
set var_distance_from_line 40
set var_distance_from_ruler 40
set var_gene_fill_color "black"
set var_interval_num 12.0

set tag_abc "ABC_ID_"

# for storing internal data (arrays)
global genesMatrix
global genesCoord
global genesIndex

#######################################################
# proc declarations
#######################################################

# place widget components
proc displayGUI {} {

    global coord_filename
    global matrix_filename
    global var_X_dir
    global var_X_start
    global var_X_end
    global var_Y_dir
    global var_Y_start
    global var_Y_end
    global var_cutoff
    global var_canvas_size_X
    global var_canvas_size_Y
    global var_canvas_size_X_userInput
    global var_canvas_size_Y_userInput
    global var_sizeOf_gene
    global color_scheme
    global message_text

    # place gene coords
    set myRowNum 1
    label .label_geneCoordFile -text "Gene Coordinates File:" -bg yellow3 
    entry .input_geneCoordFile -relief sunken -textvariable coord_filename -bg white -width 20
    grid .label_geneCoordFile -sticky news -row $myRowNum -column 0 -columnspan 2
    grid .input_geneCoordFile -sticky news -row $myRowNum -column 2 -columnspan 4

    # place matrix file
    incr myRowNum
    label .label_matrixFile -text "Matrix File:" -bg yellow3
    entry .input_matrixFile -relief sunken -textvariable matrix_filename -bg white -width 20
    grid .label_matrixFile -sticky news -row $myRowNum -column 0 -columnspan 2
    grid .input_matrixFile -sticky news -row $myRowNum -column 2 -columnspan 4

    # place annotation file
    incr myRowNum
    label .label_annotFile -text "Annotation File:" -bg yellow3
    entry .input_annotFile -relief sunken -textvariable annotation_file -bg white -width 20
    grid .label_annotFile -sticky news -row $myRowNum -column 0 -columnspan 2
    grid .input_annotFile -sticky news -row $myRowNum -column 2 -columnspan 4

    # place chr x file
    incr myRowNum
    label .label_X_dir -text "Chr X dir" -justify left -bg yellow3
    entry .input_X_dir -relief sunken -textvariable var_X_dir -bg white -width 5
    grid .label_X_dir -sticky news -row $myRowNum -column 0
    grid .input_X_dir -sticky news -padx 2 -row $myRowNum -column 1
    label .label_X_start -text "Start" -justify left -bg yellow3
    entry .input_X_start -relief sunken -textvariable var_X_start -bg white -width 12
    grid .label_X_start -sticky news -row $myRowNum -column 2
    grid .input_X_start -sticky news -padx 2 -row $myRowNum -column 3
    label .label_X_end -text "End" -justify left -bg yellow3
    entry .input_X_end -relief sunken -textvariable var_X_end -bg white -width 12
    grid .label_X_end -sticky news -row $myRowNum -column 4
    grid .input_X_end -sticky news -padx 2 -row $myRowNum -column 5

    # place chr y file
    incr myRowNum
    label .label_Y_dir -text "Chr Y dir" -justify left -bg yellow3
    entry .input_Y_dir -relief sunken -textvariable var_Y_dir -bg white -width 5
    grid .label_Y_dir -sticky news -row $myRowNum -column 0
    grid .input_Y_dir -sticky news -padx 2 -row $myRowNum -column 1
    label .label_Y_start -text "Start" -justify left -bg yellow3
    entry .input_Y_start -relief sunken -textvariable var_Y_start -bg white -width 12
    grid .label_Y_start -sticky news -row $myRowNum -column 2
    grid .input_Y_start -sticky news -padx 2 -row $myRowNum -column 3
    label .label_Y_end -text "End" -justify left -bg yellow3
    entry .input_Y_end -relief sunken -textvariable var_Y_end -bg white -width 12
    grid .label_Y_end -sticky news -row $myRowNum -column 4
    grid .input_Y_end -sticky news -padx 2 -row $myRowNum -column 5

    # place cutoff
    incr myRowNum
    label .label_cutoff -text "Identity cutoff:" -justify left -bg yellow3
    entry .input_cutoff -relief sunken -textvariable var_cutoff -bg white -width 5
    grid .label_cutoff -sticky news -row $myRowNum -column 0
    grid .input_cutoff -sticky news -padx 2 -row $myRowNum -column 1

    # place canvas x and y
    incr myRowNum
    label .label_canvas_size -text "Canvas Size" -justify left -bg yellow3
    grid .label_canvas_size -sticky news -row $myRowNum -column 0 -columnspan 2
    label .label_canvas_size_X -text "X:" -justify left -bg yellow3
    entry .input_canvas_size_X -relief sunken -textvariable \
	var_canvas_size_X_userInput -bg	white -width 7
    grid .label_canvas_size_X -sticky news -row $myRowNum -column 2
    grid .input_canvas_size_X -sticky news -padx 2 -row $myRowNum -column 3
    label .label_canvas_size_Y -text "Y:" -justify left -bg yellow3
    entry .input_canvas_size_Y -relief sunken -textvariable \
	var_canvas_size_Y_userInput -bg	white -width 7
    grid .label_canvas_size_Y -sticky news -row $myRowNum -column 4
    grid .input_canvas_size_Y -sticky news -padx 2 -row $myRowNum -column 5

    # place size of gene
    incr myRowNum
    label .label_sizeOf_gene -text "Size of gene\n(pixels):" -bg yellow3
    entry .input_sizeOf_gene -relief sunken -textvariable var_sizeOf_gene -bg white -width 5
    grid .label_sizeOf_gene -sticky news -row $myRowNum -column 0
    grid .input_sizeOf_gene -sticky ew -padx 2 -row $myRowNum -column 1

    # place load button and canvas button
    incr myRowNum
    button .startup1 -text "1. Load\nData" -command {read_genome_data} -bg orange
    grid .startup1 -sticky news -row $myRowNum -column 0
    button .startup2 -text "2. Load\nAnnotation" -command {read_annotation_data} -bg orange
    grid .startup2 -sticky news -row $myRowNum -column 1
    button .startup3 -text " 3. Plot \n Canvas " -command {calculateCanvasVars} -bg orange
    grid .startup3 -sticky news -row $myRowNum -column 2

    # message box
    label .message_box -text $message_text -bg blue4 -fg gray
    grid .message_box -sticky news -row $myRowNum -column 3 -columnspan 3

    # place postscript button
    incr myRowNum
    button .ps_button -text "Make PostScript" -command {make_PostScript} -bg gray
    grid .ps_button -sticky news -row $myRowNum -column 0
    global var_post_script_file
    entry .input_post_script_file -relief sunken -textvariable var_post_script_file -bg white
    grid .input_post_script_file -sticky news -row $myRowNum -column 1 -columnspan 2

    # place node painter button and canvas editor button
    incr myRowNum
    button .paint_button -text "Node Painter" -command {Node_Painter} -bg gray
    grid .paint_button -sticky news -row $myRowNum -column 0
    button .editor_button -text "Canvas Editor" -command {Canvas_Editor} -bg gray
    grid .editor_button -sticky news -row $myRowNum -column 1

    label .global_configuration_l -text "Color Mode: " -bg "lightblue"
    radiobutton .global_configuration_b1 -relief sunken -variable color_scheme \
            -text " White " -value "WHITE"
    radiobutton .global_configuration_b2 -relief sunken -variable color_scheme \
            -text " Black " -value "BLACK"
    grid .global_configuration_l -row $myRowNum -column 2
    grid .global_configuration_b1 -row $myRowNum -column 3
    # grid .global_configuration_b2 -row $myRowNum -column 4

    # place help, about, exit buttons
    incr myRowNum

    grid .global_configuration_b2 -row $myRowNum -column 3

    button .help_button -text "Help" -command {HowTo_PhyloWebify} -bg gray
    grid .help_button -sticky news -row $myRowNum -column 0
    button .about_button -text "About" -command {show_about} -bg gray
    grid .about_button -sticky news -row $myRowNum -column 1
    button .exit_button -text "Exit" -command {exit} -bg gray
    grid .exit_button -sticky news -row $myRowNum -column 5
}

# =====================================================

proc read_genome_data {} {

    global coord_filename
    global matrix_filename

    if { [file exists $matrix_filename] && [file exists $coord_filename] } {
	readData
	puts "Read Data Done"
	puts " "
	bell
    } else {
	set choice [tk_messageBox -type	ok -message "\nbad input file\n" -icon error]
    }
}

# =====================================================

proc read_annotation_data {} {

    global annotation_file

    if { [file exists $annotation_file] } {
	readAnnotation
	puts "Read Annotation Done"
	puts " "
	bell
    } else {
	set choice [tk_messageBox -type	ok -message "\nbad input file\n" -icon error]
    }

}

# =====================================================

proc createCanvas {} {

    global var_canvas_size_X
    global var_canvas_size_Y
    global color_scheme
    global message_text

    if {$color_scheme == "WHITE"} {
	set canvas_background "white"
    }

    if {$color_scheme == "BLACK"} {
	set canvas_background "black"
    }

    set message_text "Plotting"
    .message_box config -text $message_text -bg blue4 -fg gray
    update

    destroy .canvasWin
    toplevel .canvasWin
    wm title .canvasWin "GenomePixelizer 2D Graph"
    wm protocol .canvasWin WM_DELETE_WINDOW "destroy .canvasWin"
    set myX [expr $var_canvas_size_X + 60]
    set myY [expr $var_canvas_size_Y + 60]
    Scrolled_Canvas .canvasWin.canvasFrame -width 800 -height 600 -bg $canvas_background \
	-scrollregion "0 0 $myX $myY"
    pack .canvasWin.canvasFrame -fill both -expand true
    
    update

    Show_Gene_ID
    Annot_Window

    drawTheLines .canvasWin.canvasFrame.canvas
    drawTheRuler .canvasWin.canvasFrame.canvas
    drawTheDots .canvasWin.canvasFrame.canvas

    set message_text "Canvas Created"
    .message_box config -text $message_text -bg blue4 -fg gray
    update

}

# =====================================================

proc show_help {} {

    destroy .win_help
    toplevel .win_help

    wm title .win_help "Help" 
    wm protocol .win_help WM_DELETE_WINDOW "destroy .win_help"
    # wm minsize .win_help 300 400

    set help_content "

INPUT:
==========================

     Gene Coordinates File:
  <group><tab><nodeName><tab><coordinate><tab><W|C><tab><color>
  example:

1\tA\t5.2\tW\tred
1\tB\t7.3\tC\tblue
2\tG\t2.4\tW\tblack

     Matrix File:

  <node1><tab><node2><tab><strength>
  example:

A\tB\t0.5
B\tG\t0.7
G\tA\t0.24

OUTPUT:
==========================

A diagonal graph will be plotted. The X axis represents the first group 
of elements, the Y axis represents the second group of elements in the 
\"Coords\" file respectively. For each relation pair defined in the matrix 
file, if the group numbers inputed by the user match the group numbers of 
the two nodes, and if the strength value is larger than the cutoff value, 
a dot will be plotted on the graph. Otherwise that specific relation pair 
will be ignored. A dot drawn at the inside of the thin line means the 
corresponding node has been used. A dot drawn at the outside of the thin 
line means the corresponding node has never been used. 

"

    label .win_help.label_help_content -text $help_content -justify left
    grid .win_help.label_help_content -sticky news

    button .win_help.button_help_content -text "Dismiss" -command {destroy .win_help}
    grid .win_help.button_help_content
}

# =====================================================

proc show_about {} {

    destroy .win_about
    toplevel .win_about

    wm title .win_about "About" 
    wm protocol .win_about WM_DELETE_WINDOW "destroy .win_about"
    # wm minsize .win_about 300 300

    set about_content "
GenoPix2D (Diagonal Plot Visualization Tool)

Copyright (C) 2003 Alexander Kozik and Brian Chan
University of California at Davis
Department of Vegetable Crops, R.Michelmore lab.
    
This program is free software. You can redistribute it and/or modify it 
under the terms of the GNU General Public License ( http://www.gnu.org ) 
as published by the Free Software Foundation; either version 2 of the 
License, or (at your option) any later version.

In other words, you are free to modify, copy, or redistribute this source 
code and its documentation in any way you like, but you must distribute 
all derivative versions as free software under the same terms that we 
provided our code to you (i.e. the GNU General Public License). This 
precludes any use of the code in proprietary or commercial software 
unless your source code is made freely available.

This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
See the GNU General Public License for more details. 

You should have received a copy of the GNU General Public License along 
with this GenoPix2D release, in the file LICENSE; if not, write to the 
Free Software Foundation, Inc., 675 Mass. Ave, Cambridge, MA 02139 USA.
"

    label .win_about.label_about_content -text $about_content -justify center
    grid .win_about.label_about_content -sticky news

    button .win_about.button_about_content -text "OK" -command {destroy .win_about}
    grid .win_about.button_about_content 

}

# =====================================================

proc drawTheDots { can } {

    global var_margin_X
    global var_margin_Y
    global var_canvas_size_X
    global var_canvas_size_Y
    global genesMatrix
    global genesCoord
    global genesIndex
    global var_cutoff
    global var_canvas_size_X_userInput
    global var_canvas_size_Y_userInput
    global var_sizeOf_gene

    global chromosome_length_X
    global chromosome_length_Y
    global chromosome_length
    global var_X_dir
    global var_Y_dir

    global graph_X_factor
    global graph_Y_factor

    global dummy_update_counter1
    global dummy_update_counter2

    global var_X_start
    global var_X_end
    global var_Y_start
    global var_Y_end

    global message_text

    if { $var_X_end == "END" } {
	set upper_limitX $chromosome_length_X
    } else {
	set upper_limitX $var_X_end
    }

    if { $var_Y_end == "END" } {
	set upper_limitY $chromosome_length_Y
    } else {
	set upper_limitY $var_Y_end
    }

    set lower_limitX $var_X_start
    set lower_limitY $var_Y_start

    set dummy_update_counter1 0
    set dummy_update_counter2 0

    # set up a bool array, indicate whether gene has been plotted
    foreach { index data } [ array get genesCoord ] {
	set plottedArray($index) 0
    }

    # draw the plotted dots (and erase the corresponding dots)
    foreach { index data } [array get genesMatrix] {

	####   MATRIX  DATA   ####
	set gene1 [lindex $data	0]
	set gene2 [lindex $data	1]
	set strength [lindex $data 2]
	#### X - Y DIRECTIONS ####
	set dir1 [lindex $genesCoord($gene1) 0]
	set dir2 [lindex $genesCoord($gene2) 0]
	# if dir's match, plot it
	if { $dir1 == $var_X_dir && $dir2 == $var_Y_dir } {
	    set matrix_dir "FOR"
	    set coord1 [lindex $genesCoord($gene1) 1]
	    set coord2 [lindex $genesCoord($gene2) 1]
	    if { $strength >= $var_cutoff && $coord1 >= $lower_limitX && $coord1 <= $upper_limitX && $coord2 >= $lower_limitY && $coord2 <= $upper_limitY } {
		# plot out the point in the 2D graph
		set coord1_p [expr $coord1 - $var_X_start]
		set coord2_p [expr $coord2 - $var_Y_start]
		Plot_Current_Dot $gene1 $coord1 $gene2 $coord2 $graph_X_factor $graph_Y_factor $can $strength $matrix_dir $coord1_p $coord2_p
		set plottedArray($gene1) 1
		set plottedArray($gene2) 1
	    }
	}
	if { $dir1 == $var_Y_dir && $dir2 == $var_X_dir } {
	    set matrix_dir "REV"
	    set coord2 [lindex $genesCoord($gene1) 1]
	    set coord1 [lindex $genesCoord($gene2) 1]
	    if { $strength >= $var_cutoff && $coord1 >= $lower_limitX && $coord1 <= $upper_limitX && $coord2 >= $lower_limitY && $coord2 <= $upper_limitY } {
		# plot out the point in the 2D graph
		set coord1_p [expr $coord1 - $var_X_start]
		set coord2_p [expr $coord2 - $var_Y_start]
		Plot_Current_Dot $gene1 $coord1 $gene2 $coord2 $graph_X_factor $graph_Y_factor $can $strength $matrix_dir $coord1_p $coord2_p
		set plottedArray($gene1) 1
		set plottedArray($gene2) 1
	    }    
	}
    }

    # plot the unplotted on the axis

    set dummy_update_counter1 0
    set dummy_update_counter2 0

    foreach { gene plotted } [array get plottedArray] {
	if { $plotted == 0 } {
	    set dir   [lindex $genesCoord($gene) 0]
	    set coord [lindex $genesCoord($gene) 1]
	    if { $dir == $var_X_dir && $coord >= $lower_limitX && $coord <= $upper_limitX } {
		set coord_p [expr $coord - $var_X_start]
		Plot_Current_Dot_on_axis $gene $coord $gene $coord $graph_X_factor $graph_Y_factor $can "1.0" "N" "X" $coord_p
		if { $dummy_update_counter1 == 100 } {
		    update
		    set dummy_update_counter1 0
		    incr dummy_update_counter2
		    puts "$dummy_update_counter2 X 100 - genes on axes"
		    set message_text "$dummy_update_counter2 X 100 - genes on axes"
		    .message_box config -text $message_text -bg blue4 -fg gray
		}
		# update
	    }
	    if { $dir == $var_Y_dir && $coord >= $lower_limitY && $coord <= $upper_limitY } {
		set coord_p [expr $coord - $var_Y_start]
		Plot_Current_Dot_on_axis $gene $coord $gene $coord $graph_X_factor $graph_Y_factor $can "1.0" "N" "Y" $coord_p
		if { $dummy_update_counter1 == 100 } {
		    update
		    set dummy_update_counter1 0
		    incr dummy_update_counter2
		    puts "$dummy_update_counter2 X 100 - genes on axes"
		    set message_text "$dummy_update_counter2 X 100 - genes on axes"
		    .message_box config -text $message_text -bg blue4 -fg gray
		}
		# update
	    }
	}
	if { $plotted == 1 } {
	    set dir   [lindex $genesCoord($gene) 0]
	    set coord [lindex $genesCoord($gene) 1]
	    if { $dir == $var_X_dir && $coord >= $lower_limitX && $coord <= $upper_limitX } {
		set coord_p [expr $coord - $var_X_start]
		Plot_Current_Dot_on_axis $gene $coord $gene $coord $graph_X_factor $graph_Y_factor $can "1.0" "Y" "X" $coord_p
		if { $dummy_update_counter1 == 100 } {
		    update
		    set dummy_update_counter1 0
		    incr dummy_update_counter2
		    puts "$dummy_update_counter2 X 100 - genes on axes"
		    set message_text "$dummy_update_counter2 X 100 - genes on axes"
		    .message_box config -text $message_text -bg blue4 -fg gray
		}
		# update
	    }
	    if { $dir == $var_Y_dir  && $coord >= $lower_limitY && $coord <= $upper_limitY } {
		set coord_p [expr $coord - $var_Y_start]
		Plot_Current_Dot_on_axis $gene $coord $gene $coord $graph_X_factor $graph_Y_factor $can "1.0" "Y" "Y" $coord_p
		if { $dummy_update_counter1 == 100 } {
		    update
		    set dummy_update_counter1 0
		    incr dummy_update_counter2
		    puts "$dummy_update_counter2 X 100 - genes on axes"
		    set message_text "$dummy_update_counter2 X 100 - genes on axes"
		    .message_box config -text $message_text -bg blue4 -fg gray
		}
		# update
	    }
	}
    }
    unset plottedArray
}

# =====================================================

proc Plot_Current_Dot { gene1 coord1 gene2 coord2 graph_X_factor graph_Y_factor can strength matrix_dir coord1_p coord2_p } {

    global var_margin_X
    global var_margin_Y
    global var_sizeOf_gene
    global var_gene_fill_color
    global var_distance_from_line
    global var_distance_from_ruler
    global var_gradient_plot

    global dummy_update_counter1
    global dummy_update_counter2

    global curX
    global curY

    global tag_abc

    global color_scheme
    global message_text

    ### WHITE COLOR MODE ###
    if {$color_scheme == "WHITE"} {
	### COLOR GRADIENT PROPORTIONAL TO IDENTITY LEVEL ###
	if { $var_gradient_plot == "Yes" } {
	    if { $strength > 0.9 && $strength <= 1.0 } {
		set var_gene_fill_color "black"
		set color_tag "black"
	    }
	    if { $strength > 0.8 && $strength <= 0.9 } {
		set var_gene_fill_color "gray10"
		set color_tag "gray10"
	    }
	    if { $strength > 0.7 && $strength <= 0.8 } {
		set var_gene_fill_color "gray20"
		set color_tag "gray20"
	    }
	    if { $strength > 0.6 && $strength <= 0.7 } {
		set var_gene_fill_color "gray30"
		set color_tag "gray30"
	    }
	    if { $strength > 0.5 && $strength <= 0.6 } {
		set var_gene_fill_color "gray40"
		set color_tag "gray40"
	    }
	    if { $strength > 0.4 && $strength <= 0.5 } {
		set var_gene_fill_color "gray50"
		set color_tag "gray50"
	    }
	    if { $strength <= 0.4 } {
		set var_gene_fill_color "gray60"
		set color_tag "gray60"
	    }
	}
    }

    ### BLACK COLOR MODE ###
    if {$color_scheme == "BLACK"} {
	### COLOR GRADIENT PROPORTIONAL TO IDENTITY LEVEL ###
	if { $var_gradient_plot == "Yes" } {
	    if { $strength > 0.9 && $strength <= 1.0 } {
		set var_gene_fill_color "white"
		set color_tag "black"
	    }
	    if { $strength > 0.8 && $strength <= 0.9 } {
		set var_gene_fill_color "gray90"
		set color_tag "gray10"
	    }
	    if { $strength > 0.7 && $strength <= 0.8 } {
		set var_gene_fill_color "gray80"
		set color_tag "gray20"
	    }
	    if { $strength > 0.6 && $strength <= 0.7 } {
		set var_gene_fill_color "gray70"
		set color_tag "gray30"
	    }
	    if { $strength > 0.5 && $strength <= 0.6 } {
		set var_gene_fill_color "gray60"
		set color_tag "gray40"
	    }
	    if { $strength > 0.4 && $strength <= 0.5 } {
		set var_gene_fill_color "gray50"
		set color_tag "gray50"
	    }
	    if { $strength <= 0.4 } {
		set var_gene_fill_color "gray40"
		set color_tag "gray60"
	    }
	}
    }

    #######     END OF COLOR GRADIENT    #######

    set x_start [expr $var_margin_X + $var_distance_from_line + $var_distance_from_ruler + $coord1_p * $graph_X_factor]
    set x_end [expr $x_start + $var_sizeOf_gene]
    set y_start [expr $var_margin_Y + $var_distance_from_line + $var_distance_from_ruler + $coord2_p * $graph_Y_factor]
    set y_end [expr $y_start + $var_sizeOf_gene]
    if { $gene1 == $gene2 } {
	# $can create rect $x_start $y_start $x_end $y_end -fill $var_gene_fill_color -tag $gene1
	update
	bell
	puts "$gene1 -=O=- $gene2 WARNING! IDENTICAL IDs"
	after 1000
    } else {
	# draw a dot at the graph
	if { $matrix_dir == "FOR" } {
	    set xy_tag $tag_abc$gene1$tag_abc$gene2
	    $can create rect $x_start $y_start $x_end $y_end -fill $var_gene_fill_color \
	    -outline $var_gene_fill_color -tag $xy_tag
	    $can bind $xy_tag <Button-1> "showName $gene1 $gene2 $coord1 $coord2 .idlabel.geneidx .idlabel.geneidy .idlabel.geneidCoordx .idlabel.geneidCoordy"
	    $can bind $xy_tag <Control-Button-1> "Print_Gene_ID $gene1 $gene2"
	    $can bind $xy_tag <Enter> {
		set curX [expr int(round([.canvasWin.canvasFrame.canvas canvasx %x]))]
		set curY [expr int(round([.canvasWin.canvasFrame.canvas canvasy %y]))]
	    }
	}
	if { $matrix_dir == "REV" } {
	    set xy_tag $tag_abc$gene2$tag_abc$gene1
	    $can create rect $x_start $y_start $x_end $y_end -fill $var_gene_fill_color \
	    -outline $var_gene_fill_color -tag $xy_tag
	    $can bind $xy_tag <Button-1> "showName $gene2 $gene1 $coord1 $coord2 .idlabel.geneidx .idlabel.geneidy .idlabel.geneidCoordx .idlabel.geneidCoordy"
	    $can bind $xy_tag <Control-Button-1> "Print_Gene_ID $gene2 $gene1"
	    $can bind $xy_tag <Enter> {
		set curX [expr int(round([.canvasWin.canvasFrame.canvas canvasx %x]))]
		set curY [expr int(round([.canvasWin.canvasFrame.canvas canvasy %y]))]
	    }
	}
	$can addtag $color_tag withtag $xy_tag
	$can addtag $tag_abc$gene1 withtag $xy_tag
	$can addtag $tag_abc$gene2 withtag $xy_tag
	# $can lower $xy_tag
	if { $dummy_update_counter1 == 100 } {
	    update
	    set dummy_update_counter1 0
	    incr dummy_update_counter2
	    puts "$dummy_update_counter2 X 100 - plotted"
	    set message_text "$dummy_update_counter2 X 100 - plotted"
	    .message_box config -text $message_text -bg blue4 -fg gray
	}
	# puts "$gene1 -=X=- $gene2 ::: $strength"
    }
    incr dummy_update_counter1
}

# =====================================================

proc Plot_Current_Dot_on_axis { gene1 coord1 gene2 coord2 graph_X_factor graph_Y_factor can strength used axis coord_p } {

    global var_margin_X
    global var_margin_Y
    global var_sizeOf_gene
    global var_gene_fill_color
    global var_distance_from_line
    global var_distance_from_ruler
    global var_gradient_plot
    global color_scheme

    global curX
    global curY

    global dummy_update_counter1

    global tag_abc
    global message_text

    if {$color_scheme == "WHITE"} {
	set gene_fill_color "white"
	set gene_outline_color "black"
    }
    if {$color_scheme == "BLACK"} {
	set gene_fill_color "black"
	set gene_outline_color "white"
    }

    set x_start [expr $var_margin_X + $var_distance_from_line + $var_distance_from_ruler + $coord_p * $graph_X_factor]
    set x_end [expr $x_start + $var_sizeOf_gene]
    set y_start [expr $var_margin_Y + $var_distance_from_line + $var_distance_from_ruler + $coord_p * $graph_Y_factor]
    set y_end [expr $y_start + $var_sizeOf_gene]

    set unplotted_X_start [expr $var_distance_from_line + $var_margin_Y - $var_sizeOf_gene -1]
    set unplotted_X_end [expr $var_distance_from_line + $var_margin_Y -1]
    set unplotted_Y_start [expr $var_distance_from_line + $var_margin_X - $var_sizeOf_gene -1]
    set unplotted_Y_end [expr $var_distance_from_line + $var_margin_X -1]
    set plotted_X_start [expr $var_distance_from_line + $var_margin_Y +1]
    set plotted_X_end [expr $var_distance_from_line + $var_margin_Y + $var_sizeOf_gene +1]
    set plotted_Y_start [expr $var_distance_from_line + $var_margin_X +1]
    set plotted_Y_end [expr $var_distance_from_line + $var_margin_X + $var_sizeOf_gene +1]

    set linear_plot_tagX "linearPlotTagX"
    set linear_plot_tagY "linearPlotTagY"

    if { $used == "Y" } {
	# have been used in the graph
	if { $axis == "X" } {
	    $can create rect $x_start $plotted_Y_start $x_end $plotted_Y_end \
		-fill $gene_fill_color -outline $gene_outline_color \
		-tag $linear_plot_tagX$gene1
	    $can addtag $tag_abc$gene1 withtag $linear_plot_tagX$gene1
	    $can addtag "plotted" withtag $linear_plot_tagX$gene1
	    $can bind $linear_plot_tagX$gene1 <Button-1> "showName $gene1 plotted $coord1 plotted .idlabel.geneidx .idlabel.geneidy .idlabel.geneidCoordx .idlabel.geneidCoordy"
	    $can bind $linear_plot_tagX$gene1 <Control-Button-1> "Print_Gene_ID $gene1 just_one"
	    $can bind $linear_plot_tagX$gene1 <Enter> {
		set curX [expr int(round([.canvasWin.canvasFrame.canvas canvasx %x]))]
		set curY [expr int(round([.canvasWin.canvasFrame.canvas canvasy %y]))]
	    }
	    incr dummy_update_counter1
	}
	if { $axis == "Y" } {
	    $can create rect $plotted_X_start $y_start $plotted_X_end $y_end \
		-fill $gene_fill_color -outline $gene_outline_color \
		-tag $linear_plot_tagY$gene2
	    $can addtag $tag_abc$gene2 withtag $linear_plot_tagY$gene2
	    $can addtag "plotted" withtag $linear_plot_tagY$gene2
	    $can bind $linear_plot_tagY$gene2 <Button-1> "showName plotted $gene2 plotted $coord2 .idlabel.geneidx .idlabel.geneidy .idlabel.geneidCoordx .idlabel.geneidCoordy"
	    $can bind $linear_plot_tagY$gene2 <Control-Button-1> "Print_Gene_ID $gene2 just_one"
	    $can bind $linear_plot_tagY$gene2 <Enter> {
		set curX [expr int(round([.canvasWin.canvasFrame.canvas canvasx %x]))]
		set curY [expr int(round([.canvasWin.canvasFrame.canvas canvasy %y]))]
	    }
	    incr dummy_update_counter1
	}
    }
    if { $used == "N" } {
	# haven't been used in the graph
	if { $axis == "X" } {
	    $can create rect $x_start $unplotted_Y_start $x_end $unplotted_Y_end \
		-fill $gene_fill_color -outline $gene_outline_color \
		-tag $tag_abc$gene1
	    $can addtag "unplotted" withtag $tag_abc$gene1
	    $can bind $tag_abc$gene1 <Button-1> "showName $gene1 unplotted $coord1 unplotted .idlabel.geneidx .idlabel.geneidy .idlabel.geneidCoordx .idlabel.geneidCoordy"
	    $can bind $tag_abc$gene1 <Control-Button-1> "Print_Gene_ID $gene1 just_one"
	    $can bind $tag_abc$gene1 <Enter> {
		set curX [expr int(round([.canvasWin.canvasFrame.canvas canvasx %x]))]
		set curY [expr int(round([.canvasWin.canvasFrame.canvas canvasy %y]))]
	    }
	    incr dummy_update_counter1
	}
	if { $axis == "Y" } {
	    $can create rect $unplotted_X_start $y_start $unplotted_X_end $y_end \
		-fill $gene_fill_color -outline $gene_outline_color \
		-tag $tag_abc$gene2
	    $can addtag "unplotted" withtag $tag_abc$gene2
	    $can bind $tag_abc$gene2 <Button-1> "showName unplotted $gene2 unplotted $coord2 .idlabel.geneidx .idlabel.geneidy .idlabel.geneidCoordx .idlabel.geneidCoordy"
	    $can bind $tag_abc$gene2 <Control-Button-1> "Print_Gene_ID $gene2 just_one"
	    $can bind $tag_abc$gene2 <Enter> {
		set curX [expr int(round([.canvasWin.canvasFrame.canvas canvasx %x]))]
		set curY [expr int(round([.canvasWin.canvasFrame.canvas canvasy %y]))]
	    }
	    incr dummy_update_counter1
	}
    }
}

# =====================================================

proc calculateCanvasVars {} {

    global var_canvas_size_X
    global var_canvas_size_Y
    global var_canvas_size_X_userInput
    global var_canvas_size_Y_userInput
    global var_margin_X
    global var_margin_Y
    global var_distance_from_line
    global var_distance_from_ruler

    global chromosome_length
    global var_X_dir
    global var_Y_dir

    global var_X_start
    global var_X_end
    global var_Y_start
    global var_Y_end

    global graph_X_factor
    global graph_Y_factor

    global message_text

    if { $var_X_start == 0 && $var_X_end == "END" && $var_Y_start == 0 && $var_Y_end == "END" } {

	set chromosome_length_X $chromosome_length($var_X_dir)
	set chromosome_length_Y $chromosome_length($var_Y_dir)
	puts "Full Extraction"

    } else {

	set chromosome_length_X [expr $var_X_end - $var_X_start]
	set chromosome_length_Y [expr $var_Y_end - $var_Y_start]
	puts "Partial Extraction"

    }

    ### CALCULATE AUTOMATICALLY ###

    set graph_X_factor [expr $var_canvas_size_X_userInput / $chromosome_length_X]

    set var_canvas_size_Y_userInput [expr $graph_X_factor * $chromosome_length_Y]

    set graph_Y_factor $graph_X_factor

    ### CHECK FOR X and Y
    set var_canvas_size_X [expr $var_canvas_size_X_userInput + $var_margin_X + $var_distance_from_line + $var_distance_from_ruler]
    set var_canvas_size_Y [expr $var_canvas_size_Y_userInput + $var_margin_Y + $var_distance_from_line + $var_distance_from_ruler]

    createCanvas
    Gene_LayOut

    bell
    set message_text " -= Canvas created =- "
    .message_box config -text $message_text -bg blue4 -fg gray
    update
    update
    puts " -= Canvas created =- "
    puts " "

}

# =====================================================

proc Gene_LayOut {} {

    .canvasWin.canvasFrame.canvas raise "gray60"
    puts "0.4"
    .canvasWin.canvasFrame.canvas raise "gray50"
    puts "0.5"
    .canvasWin.canvasFrame.canvas raise "gray40"
    puts "0.6"
    .canvasWin.canvasFrame.canvas raise "gray30"
    puts "0.7"
    .canvasWin.canvasFrame.canvas raise "gray20"
    puts "0.8"
    .canvasWin.canvasFrame.canvas raise "gray10"
    puts "0.9"
    .canvasWin.canvasFrame.canvas raise "black"
    puts "1.0"
}

# =====================================================

proc Color_Reset {} {

    global color_scheme
    global message_text

    if {$color_scheme == "WHITE"} {

	.canvasWin.canvasFrame.canvas config -bg "white"
	.canvasWin.canvasFrame.canvas itemconfigure "ruler_label" -fill "black"

	.canvasWin.canvasFrame.canvas raise "gray60"
	.canvasWin.canvasFrame.canvas itemconfigure "gray60" -fill "gray60" -outline "gray60"
	puts "0.4"
	.canvasWin.canvasFrame.canvas raise "gray50"
	.canvasWin.canvasFrame.canvas itemconfigure "gray50" -fill "gray50" -outline "gray50"
	puts "0.5"
	.canvasWin.canvasFrame.canvas raise "gray40"
	.canvasWin.canvasFrame.canvas itemconfigure "gray40" -fill "gray40" -outline "gray40"
	puts "0.6"
	.canvasWin.canvasFrame.canvas raise "gray30"
	.canvasWin.canvasFrame.canvas itemconfigure "gray30" -fill "gray30" -outline "gray30"
	puts "0.7"
	.canvasWin.canvasFrame.canvas raise "gray20"
	.canvasWin.canvasFrame.canvas itemconfigure "gray20" -fill "gray20" -outline "gray20"
	puts "0.8"
	.canvasWin.canvasFrame.canvas raise "gray10"
	.canvasWin.canvasFrame.canvas itemconfigure "gray10" -fill "gray10" -outline "gray10"
	puts "0.9"
	.canvasWin.canvasFrame.canvas raise "black"
	.canvasWin.canvasFrame.canvas itemconfigure "black" -fill "black" -outline "black"
	puts "1.0"
	.canvasWin.canvasFrame.canvas raise "plotted"
	.canvasWin.canvasFrame.canvas itemconfigure "plotted" -fill "white" -outline "black"
	puts "plotted"
	.canvasWin.canvasFrame.canvas raise "unplotted"
	.canvasWin.canvasFrame.canvas itemconfigure "unplotted" -fill "white" -outline "black"
	puts "unplotted"
    }
    if {$color_scheme == "BLACK"} {

	.canvasWin.canvasFrame.canvas config -bg "black"
	.canvasWin.canvasFrame.canvas itemconfigure "ruler_label" -fill "gray75"

	.canvasWin.canvasFrame.canvas raise "gray60"
	.canvasWin.canvasFrame.canvas itemconfigure "gray60" -fill "gray40" -outline "gray40"
	puts "0.4"
	.canvasWin.canvasFrame.canvas raise "gray50"
	.canvasWin.canvasFrame.canvas itemconfigure "gray50" -fill "gray50" -outline "gray50"
	puts "0.5"
	.canvasWin.canvasFrame.canvas raise "gray40"
	.canvasWin.canvasFrame.canvas itemconfigure "gray40" -fill "gray60" -outline "gray60"
	puts "0.6"
	.canvasWin.canvasFrame.canvas raise "gray30"
	.canvasWin.canvasFrame.canvas itemconfigure "gray30" -fill "gray70" -outline "gray70"
	puts "0.7"
	.canvasWin.canvasFrame.canvas raise "gray20"
	.canvasWin.canvasFrame.canvas itemconfigure "gray20" -fill "gray80" -outline "gray80"
	puts "0.8"
	.canvasWin.canvasFrame.canvas raise "gray10"
	.canvasWin.canvasFrame.canvas itemconfigure "gray10" -fill "gray90" -outline "gray90"
	puts "0.9"
	.canvasWin.canvasFrame.canvas raise "black"
	.canvasWin.canvasFrame.canvas itemconfigure "black" -fill "white" -outline "white"
	puts "1.0"
	.canvasWin.canvasFrame.canvas raise "plotted"
	.canvasWin.canvasFrame.canvas itemconfigure "plotted" -fill "black" -outline "white"
	puts "plotted"
	.canvasWin.canvasFrame.canvas raise "unplotted"
	.canvasWin.canvasFrame.canvas itemconfigure "unplotted" -fill "black" -outline "white"
	puts "unplotted"
    }
    set message_text "Color Reset Done"
    .message_box config -text $message_text -bg blue4 -fg gray
    bell
}

# =====================================================

proc drawTheRuler { can } {

    global var_margin_X
    global var_margin_Y
    global var_distance_from_line
    global var_distance_from_ruler
    global var_canvas_size_X_userInput
    global var_canvas_size_Y_userInput
    global chromosome_length
    global var_X_dir
    global var_Y_dir
    global var_interval_num

    global graph_X_factor
    global graph_Y_factor

    global var_X_start
    global var_X_end
    global var_Y_start
    global var_Y_end

    global color_scheme

    if {$color_scheme == "WHITE"} {
	set text_color "black"
    }
    if {$color_scheme == "BLACK"} {
	set text_color "gray75"
    }

    if { $var_X_start == 0 && $var_X_end == "END" && $var_Y_start == 0 && $var_Y_end == "END" } {
	set chromosome_length_X $chromosome_length($var_X_dir)
	set chromosome_length_Y $chromosome_length($var_Y_dir)
	# puts "Full Extraction"
    } else {
	set chromosome_length_X [expr $var_X_end - $var_X_start]
	set chromosome_length_Y [expr $var_Y_end - $var_Y_start]
	# puts "Partial Extraction"
    }

    set myMaxX $chromosome_length_X
    set myMaxY $chromosome_length_Y

    if { $myMaxX >= $myMaxY } {
	set myMaxValue $myMaxX
	set myMaxValue_cond $myMaxX
    }
    if { $myMaxY >= $myMaxX } {
	set myMaxValue $myMaxY
	set myMaxValue_cond $myMaxY
    }

    set value_coeff 0
    if { $myMaxValue_cond <= 1000 } {
	while { $myMaxValue <= 1000 } {
	    set myMaxValue [expr $myMaxValue*10.0]
	    incr value_coeff
	}
	set value_coeff [expr pow(10,$value_coeff)]
    }
    if { $myMaxValue_cond > 1000 } {
	while { $myMaxValue >= 1000 } {
	    set myMaxValue [expr $myMaxValue/10.0]
	    incr value_coeff
	}
	set value_coeff [expr 10.000000000/(pow(10,$value_coeff))]
    }

    puts "$value_coeff - Value Coefficient"

    # draw the markers (for the X axis)
    set interval_num $var_interval_num
    set intervalX [expr ($chromosome_length_X * $graph_X_factor) / $interval_num]
    puts "$intervalX - Interval X"
    for {set i 0} {$i <= $interval_num} {incr i} {
	set myCurrMark [expr $i * $intervalX]
	set myX_end $var_margin_X
	set myX_start [expr $myX_end - 5]
	set myX [expr $var_margin_X + $var_distance_from_ruler + $var_distance_from_line + $myCurrMark + 1]
	$can create line $myX $myX_start $myX $myX_end -width 2 -fill $text_color -tag "ruler_label"
	set myCurrVal [expr round (($value_coeff*$i*$myMaxX/$interval_num) + $value_coeff*$var_X_start)]
	$can create text $myX 7 -text $myCurrVal -anchor n -fill $text_color -tag "ruler_label"
    }
    # draw the markers (for the Y axis)
    set interval_num $var_interval_num
    set intervalY [expr ($chromosome_length_Y * $graph_Y_factor) / $interval_num]
    puts "$intervalY - Interval Y"
    for {set i 0} {$i <= $interval_num} {incr i} {
	set myCurrMark [expr $i * $intervalY]
	set myY_end $var_margin_Y
	set myY_start [expr $myY_end - 5]
	set myY [expr $var_margin_Y + $var_distance_from_ruler + $var_distance_from_line + $myCurrMark + 1]
	$can create line $myY_start $myY $myY_end $myY -width 2 -fill $text_color -tag "ruler_label"
	set myCurrVal [expr round (($value_coeff*$i*$myMaxY/$interval_num) + $value_coeff*$var_Y_start)]
	$can create text 7 $myY -text $myCurrVal -anchor w -fill $text_color -tag "ruler_label"
    }
}

# =====================================================

proc drawTheLines { can	} {

    global var_margin_X
    global var_margin_Y
    global var_canvas_size_X
    global var_canvas_size_Y
    global var_distance_from_line
    global var_distance_from_ruler

    global chromosome_length_X
    global chromosome_length_Y

    global graph_X_factor
    global graph_Y_factor

    global var_canvas_size_X_userInput
    global var_canvas_size_Y_userInput

    global chromosome_length
    global var_X_dir
    global var_Y_dir

    global var_X_start
    global var_X_end
    global var_Y_start
    global var_Y_end

    global color_scheme

    if {$color_scheme == "WHITE"} {
	set text_color "black"
    }
    if {$color_scheme == "BLACK"} {
	set text_color "gray75"
    }

    if { $var_X_start == 0 && $var_X_end == "END" && $var_Y_start == 0 && $var_Y_end == "END" } {
	set chromosome_length_X $chromosome_length($var_X_dir)
	set chromosome_length_Y $chromosome_length($var_Y_dir)
	# puts "Full Extraction"
    } else {
	set chromosome_length_X [expr $var_X_end - $var_X_start]
	set chromosome_length_Y [expr $var_Y_end - $var_Y_start]
	# puts "Partial Extraction"
    }

    set x_start [expr $var_margin_X + $var_distance_from_line + $var_distance_from_ruler]
    set x_end [expr $graph_X_factor*$chromosome_length_X + $var_margin_X + $var_distance_from_line + $var_distance_from_ruler]
    set y $var_margin_X
    $can create line $x_start $y $x_end $y -width 2 -tags "xline ruler_label" -fill $text_color
    $can create line $x_start [expr $y + $var_distance_from_ruler] $x_end \
	[expr $y + $var_distance_from_ruler] -width 1 -tag "xline" -fill "gray"

    set y_start [expr $var_margin_Y + $var_distance_from_line + $var_distance_from_ruler]
    set y_end [expr $graph_Y_factor*$chromosome_length_Y + $var_margin_Y + $var_distance_from_line + $var_distance_from_ruler]
    set x $var_margin_Y
    $can create line $x $y_start $x $y_end -width 2 -tags "yline ruler_label" -fill $text_color
    $can create line [expr $x + $var_distance_from_ruler] $y_start \
	[expr $x + $var_distance_from_ruler] $y_end -width 1 -tag "yline" -fill "gray"

}

# =====================================================

proc Scrolled_Canvas { c args }	{

    frame $c
    eval {canvas $c.canvas \
	      -xscrollcommand [list $c.xscroll set] \
	      -yscrollcommand [list $c.yscroll set] \
	      -highlightthickness 0 \
	      -borderwidth 0} $args
    scrollbar $c.xscroll -orient horizontal \
	-command [list $c.canvas xview]
    scrollbar $c.yscroll -orient vertical \
	-command [list $c.canvas yview]
    grid $c.canvas $c.yscroll -sticky news
    grid $c.xscroll -sticky ew
    grid rowconfigure $c 0 -weight 1
    grid columnconfigure $c 0 -weight 1
    return $c.canvas
}

# =====================================================

proc readAnnotation {} {

    global annotation_file
    global annotation_array
    global message_text

    set annot_fp [open $annotation_file]
    set message_text "Loading Annotation"
    .message_box config -text $message_text -bg blue4 -fg gray
    update
    set counter 1

    while { [gets $annot_fp line] >= 0 } {

	if { [string equal $line ""] } {
	    continue ; # if empty line, skip it
	}

	set sep	"\t"
	set gene [lindex [split $line $sep] 0]
	set annot [lindex [split $line $sep] 1]

	# puts "$counter  -=-  $gene"
	incr counter
	# update
	set annotation_array($gene) $annot
    }
    set message_text "Annotation Loaded"
    .message_box config -text $message_text -bg blue4 -fg gray
    puts " "
    puts "$counter genes with annotation loaded"
    puts " "
    close $annot_fp
}

# =====================================================

proc Annot_Window {} {

    global keyword_search
    set keyword_search "search by keyword"

    destroy .annot_text
    toplevel .annot_text
    # wm transient .annot_text .canvasWin.canvasFrame.canvas
    wm title .annot_text "Annotation"
    wm minsize .annot_text 500 200
    # wm maxsize .annot_text 500 200
    wm protocol .annot_text WM_DELETE_WINDOW "destroy .annot_text"

    eval {text .annot_text.info -wrap word -bg "white" \
           -xscrollcommand [list .annot_text.xscroll set] \
           -yscrollcommand [list .annot_text.yscroll set]}
    scrollbar .annot_text.xscroll -orient horizontal \
              -command [list .annot_text.info xview]
    scrollbar .annot_text.yscroll -orient vertical \
              -command [list .annot_text.info yview]
    grid .annot_text.info .annot_text.yscroll -sticky news
    grid .annot_text.xscroll -sticky news
    grid rowconfigure .annot_text 0 -weight 1
    grid columnconfigure .annot_text 0 -weight 1

    entry .annot_text.entry_search -bg lightgreen -width 42 -textvariable keyword_search
    grid .annot_text.entry_search
    bind .annot_text.entry_search <Return> {Search_by_Keyword $keyword_search}
}

proc Search_by_Keyword { keyword_search } {

    global annotation_array
    global tag_abc
    global graph_fill_color
    global graph_outline_color

    puts $keyword_search

    set pattern_find "FALSE"

    set is_it_here1 [winfo exists .c_editor]
    if {$is_it_here1 == "0"} {
        Canvas_Editor
    }

    set is_it_there1 [winfo exists .annot_text]
    if {$is_it_there1 == "0"} {
        Annot_Window
    }

    .annot_text.info delete 1.0 end
    .annot_text.info insert end " --- Results of Search by Keyword: $keyword_search --- \n"

    foreach { gene annotation } [array get annotation_array] {
	set match_query [string match -nocase "*$keyword_search\*" $annotation]
	# puts "$gene\t$annotation"
	if { $match_query == 1 } {
	    set pattern_find "TRUE"
	    # puts "$gene\t$annotation"
	    set canvas_query [.canvasWin.canvasFrame.canvas find withtag $tag_abc$gene]
	    if { $canvas_query != "" } {
		puts "$gene\t$annotation"
		.annot_text.info insert end "$gene\t$annotation\n"
		.canvasWin.canvasFrame.canvas itemconfigure $tag_abc$gene -fill $graph_fill_color -outline $graph_outline_color
		.canvasWin.canvasFrame.canvas raise $tag_abc$gene
		update
	    }
	}
    }

    if { $pattern_find == "FALSE" } {
	puts "No match for $keyword_search"
	.annot_text.info insert end " --- No match --- \n"
    }
    bell
}

# =====================================================

proc readData {} {

    global coord_filename
    global matrix_filename
    global genesMatrix
    global genesCoord
    global genesIndex
    global chromosome_length
    global message_text

    set genesMatrix(1) "first_step"
    unset genesMatrix
    set genesCoord(1) "first_step"
    unset genesCoord

    set gene_counter 0
    set message_text "Loading Coordinates"
    .message_box config -text $message_text -bg blue4 -fg gray
    update

    puts "Coords loading ... please wait"

    # read all the coord data, and store them
    set coord_fp [open $coord_filename]
    set max_index "0"
    while { [gets $coord_fp line] >= 0 } {

	if { [string equal $line ""] } {
	    continue ; # if empty line, skip it
	}

	set sep	"\t"
	set dir [lindex [split $line $sep] 0]
	set gene [lindex [split $line $sep] 1]
	set coordinate [lindex [split $line $sep] 2]
	set wORc [lindex [split	$line $sep] 3]
	set color [lindex [split $line $sep] 4]
	lappend	data $dir $coordinate $wORc $color
	set genesCoord($gene) $data
	# puts $gene
	unset data
	incr gene_counter

	if { [string compare $max_index $dir] == -1 } {
	  set chromosome_length($dir) 0
	  set max_index $dir
	}

	if { $chromosome_length($dir) < $coordinate } {
	  set chromosome_length($dir) $coordinate
	}
    }
    close $coord_fp
    puts " "
    puts "---------------------------------------"
    puts "Coords for $gene_counter genes loaded"
    puts "---------------------------------------"
    puts " "
    puts "Matrix loading ... please wait"

    set message_text "Loading Matrix"
    .message_box config -text $message_text -bg blue4 -fg gray
    update

    # read all the relation data, and store them
    set index1 0
    set index2 0
    set matrix_fp [open $matrix_filename]
    while { [gets $matrix_fp line] >= 0 } {

	if { [string equal $line ""] } {
	    continue ;# if empty line, skip it
	}

	set sep	"\t"
	set gene1 [lindex [split $line $sep] 0]
	set gene2 [lindex [split $line $sep] 1]
	set strength [lindex [split $line $sep]	2]
	if { $gene1 != $gene2 } {
	    lappend data $gene1 $gene2 $strength
	    set genesMatrix($index1) $data
	    unset data
	    # puts $index1
	    incr index1
	}
	if { $gene1 == $gene2 } {
	    puts "$gene1 -=- $gene2"
	    incr index2
	    puts [expr $index1 + $index2]
	}
    }
    close $matrix_fp
    bell
    puts " "
    puts "---------------------------------------"
    puts "$index1  - GOOD PAIRS"
    puts "$index2  - PAIRS WITH IDENTICAL IDs"
    puts "---------------------------------------"
    puts " "

    set message_text ""

    foreach {key value} [array get chromosome_length] {
	puts "$key -=- $value"
    }
    bell
    set message_text "Coords and Matrix Loaded"
    .message_box config -text $message_text -bg blue4 -fg gray
    # after 1000
}

# =====================================================

proc Show_Gene_ID {} {

    destroy .idlabel
    toplevel .idlabel
    # wm transient .idlabel .canvasWin.canvasFrame.canvas
    wm title .idlabel "Gene IDs"
    wm protocol .idlabel WM_DELETE_WINDOW "destroy .idlabel"

    label .idlabel.wc1_l -text "WC" -justify right -bg gray
    entry .idlabel.wc1_e -relief sunken -bg lightblue -width 3
    label .idlabel.labelx -text "Gene ID A:" -justify right -bg gray
    entry .idlabel.geneidx -relief sunken -textvariable geneNameX -bg lightblue
    label .idlabel.labelCoordx -text "Coord A:" -justify right -bg gray
    entry .idlabel.geneidCoordx -relief sunken -textvariable geneCoordX -bg lightblue
    grid .idlabel.labelx .idlabel.geneidx .idlabel.labelCoordx .idlabel.geneidCoordx .idlabel.wc1_l .idlabel.wc1_e

    label .idlabel.wc2_l -text "WC" -justify right -bg gray
    entry .idlabel.wc2_e -relief sunken -bg lightblue -width 3
    label .idlabel.labely -text "Gene ID B:" -justify right -bg gray
    entry .idlabel.geneidy -relief sunken -textvariable geneNameY -bg lightblue
    label .idlabel.labelCoordy -text "Coord B:" -justify right -bg gray
    entry .idlabel.geneidCoordy -relief sunken -textvariable geneCoordY -bg lightblue
    grid .idlabel.labely .idlabel.geneidy .idlabel.labelCoordy .idlabel.geneidCoordy .idlabel.wc2_l .idlabel.wc2_e
    
    bind .idlabel.geneidx <Return> {highLightGene .canvasWin.canvasFrame.canvas $geneNameX "X"}
    bind .idlabel.geneidy <Return> {highLightGene .canvasWin.canvasFrame.canvas $geneNameY "Y"}

}

# =====================================================

proc highLightGene { can gene_id dir} {

    global tag_abc
    global var_margin_X
    global var_margin_Y
    global var_canvas_size_X
    global var_canvas_size_Y
    global var_distance_from_line
    global message_text
    global genesCoord

    set message_text ""
    .message_box config -text $message_text -bg blue4 -fg gray

    $can delete "xy_pointer"

    set gene_found "FALSE"

    set is_it_there1 [winfo exists .annot_text]
    if {$is_it_there1 == "0"} {
        Annot_Window
    }

    .annot_text.info delete 1.0 end

    if { $dir == "X" } {
	set xy_coords [$can coords linearPlotTagX$gene_id]
	puts $xy_coords
	if { $xy_coords != "" } {
	    set gene_found "TRUE"
	    set x_start [expr $var_margin_Y + $var_distance_from_line]
	    set x_end $var_canvas_size_X
	    set y_start [expr $var_margin_X + $var_distance_from_line]
	    set y_end $var_canvas_size_Y

	    set x1 [lindex $xy_coords 0]
	    puts $x1
	    set x2 [lindex $xy_coords 2]
	    puts $x2

	    Paint_Adjacent_Nodes $gene_id "X" $x1 $x2
	    $can create rect $x1 $y_start $x2 $y_end -fill "" -outline "lightgreen" -tag "xy_pointer"
	    $can lower "xy_pointer"
	}
    }

        if { $dir == "Y" } {
	set xy_coords [$can coords linearPlotTagY$gene_id]
	puts $xy_coords
	if { $xy_coords != "" } {
	    set gene_found "TRUE"
	    set x_start [expr $var_margin_Y + $var_distance_from_line]
	    set x_end $var_canvas_size_X
	    set y_start [expr $var_margin_X + $var_distance_from_line]
	    set y_end $var_canvas_size_Y

	    set y1 [lindex $xy_coords 1]
	    puts $y1
	    set y2 [lindex $xy_coords 3]
	    puts $y2

	    Paint_Adjacent_Nodes $gene_id "Y" $y1 $y2
	    $can create rect $x_start $y1 $x_end $y2 -fill "" -outline "lightgreen" -tag "xy_pointer"
	    $can lower "xy_pointer"
	}
    }

    if { $gene_found == "TRUE" } {
	bell
	puts "Gene $gene_id is plotted on $dir axis"
    }
    if { $gene_found == "FALSE" } {
	bell
	puts "Gene $gene_id is not plotted on $dir axis"
    }
}

# =====================================================

proc showName {c1 c2 c3 c4 layer1 layer2 layer3 layer4} {

    global annotation_array
    global var_margin_X
    global var_margin_Y
    global var_canvas_size_X
    global var_canvas_size_Y
    global var_distance_from_line
    global var_distance_from_ruler
    global genesCoord

    global tag_abc
    global message_text

    set message_text ""
    .message_box config -text $message_text -bg blue4 -fg gray

    set is_it_there1 [winfo exists .annot_text]
    if {$is_it_there1 == "0"} {
        Annot_Window
    }

    set is_it_there2 [winfo exists .idlabel]
    if {$is_it_there2 == "0"} {
        Show_Gene_ID
    }

    set wc1_query [info exists genesCoord($c1)]
    set wc2_query [info exists genesCoord($c2)]

    if { $wc1_query == 1 } {
	set wc1 [lindex $genesCoord($c1) 2]
    } else {
	set wc1 ""
    }
    if { $wc2_query == 1 } {
	set wc2 [lindex $genesCoord($c2) 2]
    } else {
	set wc2 ""
    }

    puts "WC: $wc1 $wc2"

    puts "$c1 -=X=- $c2"
    $layer1 delete 0 end
    $layer1 insert end $c1
    $layer2 delete 0 end
    $layer2 insert end $c2
    $layer3 delete 0 end
    $layer3 insert end $c3
    $layer4 delete 0 end
    $layer4 insert end $c4

    .idlabel.wc1_e delete 0 end
    .idlabel.wc2_e delete 0 end

    .idlabel.wc1_e insert end $wc1
    .idlabel.wc2_e insert end $wc2

    .annot_text.info delete 1.0 end
    .canvasWin.canvasFrame.canvas delete "xy_pointer"

    ################################################
    set query1 [info exists annotation_array($c1)]
    if { $query1 == 1 } {
	puts "node_A: $c1 -=- $annotation_array($c1)"
	# .annot_text.info delete 1.0 end
	.annot_text.info insert end "=======\nnode_A: $c1 -=- $annotation_array($c1)\n======="
    } else {
	puts "node_A: $c1 -=- no data"
	# .annot_text.info delete 1.0 end
	.annot_text.info insert end "=======\nnode_A: $c1 -=- No data\n======="
    }
    set query2 [info exists annotation_array($c2)]
    if { $query2 == 1 } {
	puts "node_B: $c2 -=- $annotation_array($c2)"
	# .annot_text.info delete 1.0 end
	.annot_text.info insert end "=======\nnode_B: $c2 -=- $annotation_array($c2)\n======="
    } else {
	puts "node_B: $c2 -=- no data"
	# .annot_text.info delete 1.0 end
	.annot_text.info insert end "=======\nnode_B: $c2 -=- No data\n======="
    }
    ################################################

    if { $c1 != "plotted" && $c2 != "plotted" && $c1 != "unplotted" && $c2 != "unplotted" } {

	set xy_coords [.canvasWin.canvasFrame.canvas coords $tag_abc$c1$tag_abc$c2]
	puts "$xy_coords - COORDS ON CANVAS"

	set x_start [expr $var_margin_Y + $var_distance_from_line]
	set x_end $var_canvas_size_X
	set y_start [expr $var_margin_X + $var_distance_from_line]
	set y_end $var_canvas_size_Y

	set x1 [lindex $xy_coords 0]
	# puts $x1
	set x2 [lindex $xy_coords 2]
	# puts $x2
	set y1 [lindex $xy_coords 1]
	# puts $y1
	set y2 [lindex $xy_coords 3]
	# puts $y2

	.annot_text.info insert end "\n --- All Adjacent Pairs: --- \n"

	Paint_Adjacent_Nodes $c1 "X" $x1 $x2

	.annot_text.info insert end "\n --------------------------- \n"

	Paint_Adjacent_Nodes $c2 "Y" $y1 $y2

	.annot_text.info insert end "\n --------------------------- \n"

	.canvasWin.canvasFrame.canvas create rect $x_start $y1 $x_end $y2 -fill "" -outline "lightblue" -tag "xy_pointer"
	.canvasWin.canvasFrame.canvas create rect $x1 $y_start $x2 $y_end -fill "" -outline "lightblue" -tag "xy_pointer"
	.canvasWin.canvasFrame.canvas lower "xy_pointer"
    }

    if { $c1 == "plotted" } {
	set xy_coords [.canvasWin.canvasFrame.canvas coords linearPlotTagY$c2]
	puts $xy_coords

	set x_start [expr $var_margin_Y + $var_distance_from_line]
	set x_end $var_canvas_size_X
	set y_start [expr $var_margin_X + $var_distance_from_line]
	set y_end $var_canvas_size_Y

	# set x1 [lindex $xy_coords 0]
	# puts $x1
	# set x2 [lindex $xy_coords 2]
	# puts $x2
	set y1 [lindex $xy_coords 1]
	puts $y1
	set y2 [lindex $xy_coords 3]
	puts $y2

	.annot_text.info insert end "\n --- All Adjacent Pairs: --- \n"

	Paint_Adjacent_Nodes $c2 "Y" $y1 $y2

	.annot_text.info insert end "\n --------------------------- \n"

	.canvasWin.canvasFrame.canvas create rect $x_start $y1 $x_end $y2 -fill "" -outline "lightgreen" -tag "xy_pointer"
	# .canvasWin.canvasFrame.canvas create rect $x1 $y_start $x2 $y_end -fill "" -outline "lightgreen" -tag "xy_pointer"
	.canvasWin.canvasFrame.canvas lower "xy_pointer"
    }

    if { $c2 == "plotted" } {
	set xy_coords [.canvasWin.canvasFrame.canvas coords linearPlotTagX$c1]
	puts $xy_coords

	set x_start [expr $var_margin_Y + $var_distance_from_line]
	set x_end $var_canvas_size_X
	set y_start [expr $var_margin_X + $var_distance_from_line]
	set y_end $var_canvas_size_Y

	set x1 [lindex $xy_coords 0]
	puts $x1
	set x2 [lindex $xy_coords 2]
	puts $x2
	# set y1 [lindex $xy_coords 1]
	# puts $y1
	# set y2 [lindex $xy_coords 3]
	# puts $y2

	.annot_text.info insert end "\n --- All Adjacent Pairs: --- \n"

	Paint_Adjacent_Nodes $c1 "X" $x1 $x2

	.annot_text.info insert end "\n --------------------------- \n"

	# .canvasWin.canvasFrame.canvas create rect $x_start $y1 $x_end $y2 -fill "" -outline "lightgreen" -tag "xy_pointer"
	.canvasWin.canvasFrame.canvas create rect $x1 $y_start $x2 $y_end -fill "" -outline "lightgreen" -tag "xy_pointer"
	.canvasWin.canvasFrame.canvas lower "xy_pointer"
    }
}

# =====================================================

proc Paint_Adjacent_Nodes { id dir c1 c2 } {

    global tag_abc

    global var_margin_X
    global var_margin_Y
    global var_canvas_size_X
    global var_canvas_size_Y
    global var_distance_from_line
    global var_distance_from_ruler

    set x_start [expr $var_margin_Y + $var_distance_from_line]
    set x_end $var_canvas_size_X
    set y_start [expr $var_margin_X + $var_distance_from_line]
    set y_end $var_canvas_size_Y

    set id_list [.canvasWin.canvasFrame.canvas find withtag $tag_abc$id]
    puts "$id - QUERY -\> $id_list"
    puts "========================================="
    foreach { item } $id_list {
	set tag_list [.canvasWin.canvasFrame.canvas gettags $item]
	set coord_list [.canvasWin.canvasFrame.canvas coords $item]
	set current_id [lindex $tag_list 0]
	regsub -all "$tag_abc" $current_id "\t" current_id
	regsub {linearPlotTagX} $current_id "\tX - Plot: \t" current_id
	regsub {linearPlotTagY} $current_id "\tY - Plot: \t" current_id
	# puts "========================================="
	# puts "$item ::: $coord_list ::: $tag_list"
	puts "$current_id"
	### DISPLAY PAIRS ON ANNOTATION WINDOW ###
	.annot_text.info insert end "$current_id\n"
	###                                    ###
	set x1 [lindex $coord_list 0]
	# puts "$x1 - X1 COORDS"
	set x2 [lindex $coord_list 2]
	# puts "$x2 - X2 COORDS"
	set y1 [lindex $coord_list 1]
	# puts "$y1 - Y1 COORDS"
	set y2 [lindex $coord_list 3]
	# puts "$y2 - Y2 COORDS"
	# puts "-----------------------------------------"
	# puts "$dir - DIRECTION"
	# puts "$c1 - C1 COORDS"
	# puts "$c2 - C2 COORDS"
	puts "-----------------------------------------"

	if { [lindex $tag_list 2] != "plotted" } {
	    if { $dir == "X" } {
		.canvasWin.canvasFrame.canvas create rect $x_start $y1 $x_end $y2 -fill "" -outline "khaki" -tag "xy_pointer"
		.canvasWin.canvasFrame.canvas create rect $x1 $y_start $x2 $y_end -fill "" -outline "khaki" -tag "xy_pointer"
		.canvasWin.canvasFrame.canvas lower "xy_pointer"
	    }
	    if { $dir == "Y" } {
		.canvasWin.canvasFrame.canvas create rect $x_start $y1 $x_end $y2 -fill "" -outline "khaki" -tag "xy_pointer"
		.canvasWin.canvasFrame.canvas create rect $x1 $y_start $x2 $y_end -fill "" -outline "khaki" -tag "xy_pointer"
		.canvasWin.canvasFrame.canvas lower "xy_pointer"
	    }
	}
    }
}

# =====================================================

proc make_PostScript {} {

    global var_canvas_size_X
    global var_canvas_size_Y
    global var_post_script_file
    global color_scheme
    global message_text

    set message_text "Creating PostScript"
    .message_box config -text $message_text -bg blue4 -fg gray
    update

    if {$color_scheme == "WHITE"} {
	set dummy_background_color "white"
    }
    if {$color_scheme == "BLACK"} {
	set dummy_background_color "black"
    }

    .canvasWin.canvasFrame.canvas delete "dummy_background"

    .canvasWin.canvasFrame.canvas create rect -1 -1 [expr $var_canvas_size_X + 65] [expr $var_canvas_size_Y + 65] \
	-fill $dummy_background_color -outline $dummy_background_color -tag "dummy_background"

    .canvasWin.canvasFrame.canvas lower "dummy_background"

    # .canvasWin.canvasFrame.canvas delete "xy_pointer"

    .canvasWin.canvasFrame.canvas postscript -height [expr $var_canvas_size_Y + 60] \
	-width [expr $var_canvas_size_X + 60] -x -1 -y -1 -file $var_post_script_file

    .canvasWin.canvasFrame.canvas delete "dummy_background"

    bell

    set message_text "PostScript Created"
    .message_box config -text $message_text -bg blue4 -fg gray
    update
    puts "PostScript created"

}

# =====================================================

proc Node_Painter {} {

    destroy .node_painter
    toplevel .node_painter
    wm title .node_painter "Node Painter"
    wm protocol .node_painter WM_DELETE_WINDOW "destroy .node_painter"
    wm minsize .node_painter 470 300

    global nodes_group1 
    global nodes_group2 
    global nodes_group3
    global nodes_group4 
    global nodes_group5 
    global nodes_group6
    global diagonal_group

    set nodes_group1 "./Group/subgroup_1.txt"
    set nodes_group2 "./Group/subgroup_2.txt"
    set nodes_group3 "./Group/subgroup_3.txt"
    set nodes_group4 "./Group/subgroup_4.txt"
    set nodes_group5 "./Group/subgroup_5.txt"
    set nodes_group6 "./Group/subgroup_6.txt"
    set diagonal_group "./Group/diagonal_group.txt"

    global color_group1 
    global color_group2 
    global color_group3 
    global color_group4 
    global color_group5 
    global color_group6
    global color_diagonal

    global outline_group1 
    global outline_group2 
    global outline_group3 
    global outline_group4 
    global outline_group5 
    global outline_group6
    global outline_diagonal

    set color_group1 "orange"
    set color_group2 "cyan"
    set color_group3 "lightblue"
    set color_group4 "yellow"
    set color_group5 "pink"
    set color_group6 "yellow"
    set color_diagonal "red"

    set outline_group1 "red"
    set outline_group2 "green3"
    set outline_group3 "blue"
    set outline_group4 "purple"
    set outline_group5 "violet"
    set outline_group6 "magenta"
    set outline_diagonal "purple"

    #####
    label .node_painter.group_list -text "group list:" -bg gray -width 14
    place .node_painter.group_list -x 120 -y 10

    label .node_painter.fill_color -text "fill color:" -bg gray -width 10
    place .node_painter.fill_color -x 250 -y 10

    label .node_painter.outline_color -text "outline:" -bg gray -width 8
    place .node_painter.outline_color -x 330 -y 10

    #####
    label .node_painter.select_group1_label -text "group1:" -bg gray -width 12
    place .node_painter.select_group1_label -x 10 -y 40

    entry .node_painter.select_group1_entry -relief sunken \
	    -textvariable nodes_group1 -bg lightyellow -width 20
    place .node_painter.select_group1_entry -x 105 -y 40

    entry .node_painter.color_group1_entry -relief sunken \
	    -textvariable color_group1 -bg lightyellow -width 8
    place .node_painter.color_group1_entry -x 260 -y 40

    entry .node_painter.outline_group1_entry -relief sunken \
	    -textvariable outline_group1 -bg lightyellow -width 8
    place .node_painter.outline_group1_entry -x 330 -y 40

    button .node_painter.paint_group1 -text "paint" -bg yellow3 \
	    -command {Change_Node_Color1}
    place .node_painter.paint_group1 -x 400 -y 38
    #####
    label .node_painter.select_group2_label -text "group2:" -bg gray -width 12
    place .node_painter.select_group2_label -x 10 -y 70

    entry .node_painter.select_group2_entry -relief sunken \
	    -textvariable nodes_group2 -bg lightyellow -width 20
    place .node_painter.select_group2_entry -x 105 -y 70

    entry .node_painter.color_group2_entry -relief sunken \
	    -textvariable color_group2 -bg lightyellow -width 8
    place .node_painter.color_group2_entry -x 260 -y 70

    entry .node_painter.outline_group2_entry -relief sunken \
	    -textvariable outline_group2 -bg lightyellow -width 8
    place .node_painter.outline_group2_entry -x 330 -y 70

    button .node_painter.paint_group2 -text "paint" -bg yellow3 \
	    -command {Change_Node_Color2}
    place .node_painter.paint_group2 -x 400 -y 68
    #####
    label .node_painter.select_group3_label -text "group3:" -bg gray -width 12
    place .node_painter.select_group3_label -x 10 -y 100

    entry .node_painter.select_group3_entry -relief sunken \
	    -textvariable nodes_group3 -bg lightyellow -width 20
    place .node_painter.select_group3_entry -x 105 -y 100

    entry .node_painter.color_group3_entry -relief sunken \
	    -textvariable color_group3 -bg lightyellow -width 8
    place .node_painter.color_group3_entry -x 260 -y 100

    entry .node_painter.outline_group3_entry -relief sunken \
	    -textvariable outline_group3 -bg lightyellow -width 8
    place .node_painter.outline_group3_entry -x 330 -y 100

    button .node_painter.paint_group3 -text "paint" -bg yellow3 \
	    -command {Change_Node_Color3}
    place .node_painter.paint_group3 -x 400 -y 98
    #####
    label .node_painter.select_group4_label -text "group4:" -bg gray -width 12
    place .node_painter.select_group4_label -x 10 -y 130

    entry .node_painter.select_group4_entry -relief sunken \
	    -textvariable nodes_group4 -bg lightyellow -width 20
    place .node_painter.select_group4_entry -x 105 -y 130

    entry .node_painter.color_group4_entry -relief sunken \
	    -textvariable color_group4 -bg lightyellow -width 8
    place .node_painter.color_group4_entry -x 260 -y 130

    entry .node_painter.outline_group4_entry -relief sunken \
	    -textvariable outline_group4 -bg lightyellow -width 8
    place .node_painter.outline_group4_entry -x 330 -y 130

    button .node_painter.paint_group4 -text "paint" -bg yellow3 \
	    -command {Change_Node_Color4}
    place .node_painter.paint_group4 -x 400 -y 128
    #####
    label .node_painter.select_group5_label -text "group5:" -bg gray -width 12
    place .node_painter.select_group5_label -x 10 -y 160

    entry .node_painter.select_group5_entry -relief sunken \
	    -textvariable nodes_group5 -bg lightyellow -width 20
    place .node_painter.select_group5_entry -x 105 -y 160

    entry .node_painter.color_group5_entry -relief sunken \
	    -textvariable color_group5 -bg lightyellow -width 8
    place .node_painter.color_group5_entry -x 260 -y 160

    entry .node_painter.outline_group5_entry -relief sunken \
	    -textvariable outline_group5 -bg lightyellow -width 8
    place .node_painter.outline_group5_entry -x 330 -y 160

    button .node_painter.paint_group5 -text "paint" -bg yellow3 \
	    -command {Change_Node_Color5}
    place .node_painter.paint_group5 -x 400 -y 158
    #####
    label .node_painter.select_group6_label -text "group6:" -bg gray -width 12
    place .node_painter.select_group6_label -x 10 -y 190

    entry .node_painter.select_group6_entry -relief sunken \
	    -textvariable nodes_group6 -bg lightyellow -width 20
    place .node_painter.select_group6_entry -x 105 -y 190

    entry .node_painter.color_group6_entry -relief sunken \
	    -textvariable color_group6 -bg lightyellow -width 8
    place .node_painter.color_group6_entry -x 260 -y 190

    entry .node_painter.outline_group6_entry -relief sunken \
	    -textvariable outline_group6 -bg lightyellow -width 8
    place .node_painter.outline_group6_entry -x 330 -y 190

    button .node_painter.paint_group6 -text "paint" -bg yellow3 \
	    -command {Change_Node_Color6}
    place .node_painter.paint_group6 -x 400 -y 188
    #####

    ##################
    # DIAGONAL COLOR #
    ##################

    label .node_painter.select_group7_label -text "diagonal:" -bg gray -width 12
    place .node_painter.select_group7_label -x 10 -y 220

    entry .node_painter.select_group7_entry -relief sunken \
	    -textvariable diagonal_group -bg lightyellow -width 20
    place .node_painter.select_group7_entry -x 105 -y 220

    entry .node_painter.color_group7_entry -relief sunken \
	    -textvariable color_diagonal -bg lightyellow -width 8
    place .node_painter.color_group7_entry -x 260 -y 220

    entry .node_painter.outline_group7_entry -relief sunken \
	    -textvariable outline_diagonal -bg lightyellow -width 8
    place .node_painter.outline_group7_entry -x 330 -y 220

    button .node_painter.paint_group7 -text "paint" -bg yellow3 \
	    -command {Change_Node_Color7}
    place .node_painter.paint_group7 -x 400 -y 218

    #############################################################

    button .node_painter.howto -text "HowTo" -bg gray -width 8 -command {HowTo_Paint_Nodes}
    place .node_painter.howto -x 20 -y 255

    button .node_painter.dismiss -text "Dismiss" -bg gray -width 8 -command {destroy .node_painter}
    place .node_painter.dismiss -x 140 -y 255

    button .node_painter.reset -text "Color Reset" -bg gray -width 8 -command {Color_Reset}
    place .node_painter.reset -x 260 -y 255

}

# =====================================================

proc HowTo_Paint_Nodes {} {

    destroy .how_to_paint
    toplevel .how_to_paint
    wm title .how_to_paint "HowTo Paint Nodes" 
    wm protocol .how_to_paint WM_DELETE_WINDOW "destroy .how_to_paint"

    label .how_to_paint.text -text "
To paint nodes in different colors you need
to set up lists of nodes belonging to distinct 
groups and place them in \"Group\" directory. 
By clicking \"paint\" program changes color 
of nodes in a given list.

Diagonal group is a special group. This group
should contain two columns with IDs. 
For example:

ID_A\tID_B
ID_B\tID_C

In this case all nodes with IDs: 
ID_A - ID_B or ID_B - ID_A, 
and 
ID_A - ID_C or ID_C - ID_A
will be painted.
"
    button .how_to_paint.dismiss -text "Dismiss" \
	-command "destroy .how_to_paint" -bg gray -width 8

    pack .how_to_paint.text -padx 7 -pady 7
    pack .how_to_paint.dismiss -padx 3 -pady 3
}

# =====================================================

proc Change_Node_Color1 {} {

    global nodes_group1 
    global color_group1
    global outline_group1
    global tag_abc
    global message_text

    set message_text "Painting Group 1"
    .message_box config -text $message_text -bg blue4 -fg gray
    update

    set open_current_group [open $nodes_group1 "r"]
    while {[gets $open_current_group current_line_group] >= 0} {
	.canvasWin.canvasFrame.canvas itemconfigure $tag_abc$current_line_group -fill $color_group1 -outline $outline_group1
	.canvasWin.canvasFrame.canvas raise $tag_abc$current_line_group
	update
    }
    close $open_current_group
    set message_text "Group 1 painted"
    .message_box config -text $message_text -bg blue4 -fg gray
    update
    bell
}
proc Change_Node_Color2 {} {

    global nodes_group2 
    global color_group2
    global outline_group2
    global tag_abc
    global message_text

    set message_text "Painting Group 2"
    .message_box config -text $message_text -bg blue4 -fg gray
    update

    set open_current_group [open $nodes_group2 "r"]
    while {[gets $open_current_group current_line_group] >= 0} {
	.canvasWin.canvasFrame.canvas itemconfigure $tag_abc$current_line_group -fill $color_group2 -outline $outline_group2
	.canvasWin.canvasFrame.canvas raise $tag_abc$current_line_group
	update
    }
    close $open_current_group
    set message_text "Group 2 painted"
    .message_box config -text $message_text -bg blue4 -fg gray
    update
    bell
}
proc Change_Node_Color3 {} {

    global nodes_group3 
    global color_group3
    global outline_group3
    global tag_abc
    global message_text

    set message_text "Painting Group 3"
    .message_box config -text $message_text -bg blue4 -fg gray
    update

    set open_current_group [open $nodes_group3 "r"]
    while {[gets $open_current_group current_line_group] >= 0} {
	.canvasWin.canvasFrame.canvas itemconfigure $tag_abc$current_line_group -fill $color_group3 -outline $outline_group3
	.canvasWin.canvasFrame.canvas raise $tag_abc$current_line_group
	update
    }
    close $open_current_group
    set message_text "Group 3 painted"
    .message_box config -text $message_text -bg blue4 -fg gray
    update
    bell
}
proc Change_Node_Color4 {} {

    global nodes_group4 
    global color_group4
    global outline_group4
    global tag_abc
    global message_text

    set message_text "Painting Group 4"
    .message_box config -text $message_text -bg blue4 -fg gray
    update

    set open_current_group [open $nodes_group4 "r"]
    while {[gets $open_current_group current_line_group] >= 0} {
	.canvasWin.canvasFrame.canvas itemconfigure $tag_abc$current_line_group -fill $color_group4 -outline $outline_group4
	.canvasWin.canvasFrame.canvas raise $tag_abc$current_line_group
	update
    }
    close $open_current_group
    set message_text "Group 4 painted"
    .message_box config -text $message_text -bg blue4 -fg gray
    update
    bell
}
proc Change_Node_Color5 {} {

    global nodes_group5 
    global color_group5
    global outline_group5
    global tag_abc
    global message_text

    set message_text "Painting Group 5"
    .message_box config -text $message_text -bg blue4 -fg gray
    update

    set open_current_group [open $nodes_group5 "r"]
    while {[gets $open_current_group current_line_group] >= 0} {
	.canvasWin.canvasFrame.canvas itemconfigure $tag_abc$current_line_group -fill $color_group5 -outline $outline_group5
	.canvasWin.canvasFrame.canvas raise $tag_abc$current_line_group
	update
    }
    close $open_current_group
    set message_text "Group 5 painted"
    .message_box config -text $message_text -bg blue4 -fg gray
    update
    bell
}
proc Change_Node_Color6 {} {

    global nodes_group6 
    global color_group6
    global outline_group6
    global tag_abc
    global message_text

    set message_text "Painting Group 6"
    .message_box config -text $message_text -bg blue4 -fg gray
    update

    set open_current_group [open $nodes_group6 "r"]
    while {[gets $open_current_group current_line_group] >= 0} {
	.canvasWin.canvasFrame.canvas itemconfigure $tag_abc$current_line_group -fill $color_group6 -outline $outline_group6
	.canvasWin.canvasFrame.canvas raise $tag_abc$current_line_group
	update
    }
    close $open_current_group
    set message_text "Group 6 painted"
    .message_box config -text $message_text -bg blue4 -fg gray
    update
    bell
}
proc Change_Node_Color7 {} {

    global diagonal_group
    global color_diagonal
    global outline_diagonal
    global tag_abc
    global message_text

    set message_text "Painting Diagonal Group"
    .message_box config -text $message_text -bg blue4 -fg gray
    update

    set plot_counter1 0
    set plot_counter2 0

    set open_current_group [open $diagonal_group "r"]
    while {[gets $open_current_group current_line_group] >= 0} {

	set id_A [lindex $current_line_group 0]
	set id_B [lindex $current_line_group 1]

	set current_diag_group1 $tag_abc$id_A$tag_abc$id_B
	set current_diag_group2 $tag_abc$id_B$tag_abc$id_A

	set plot_test1 [.canvasWin.canvasFrame.canvas find withtag $current_diag_group1]
	set plot_test2 [.canvasWin.canvasFrame.canvas find withtag $current_diag_group2]

	if {$plot_test1 != ""} {
	    .canvasWin.canvasFrame.canvas itemconfigure $current_diag_group1 -fill $color_diagonal -outline $outline_diagonal
	    .canvasWin.canvasFrame.canvas raise $current_diag_group1
	    incr plot_counter1
	    puts "AB ::: $id_A ::: $id_B ::: $plot_counter1"

	    .canvasWin.canvasFrame.canvas itemconfigure linearPlotTagX$id_A -fill $color_diagonal -outline $outline_diagonal
	    .canvasWin.canvasFrame.canvas itemconfigure linearPlotTagX$id_B -fill $color_diagonal -outline $outline_diagonal
	    .canvasWin.canvasFrame.canvas raise linearPlotTagX$id_A
	    .canvasWin.canvasFrame.canvas raise linearPlotTagX$id_B

	    .canvasWin.canvasFrame.canvas itemconfigure linearPlotTagY$id_A -fill $color_diagonal -outline $outline_diagonal
	    .canvasWin.canvasFrame.canvas itemconfigure linearPlotTagY$id_B -fill $color_diagonal -outline $outline_diagonal
	    .canvasWin.canvasFrame.canvas raise linearPlotTagY$id_A
	    .canvasWin.canvasFrame.canvas raise linearPlotTagY$id_B
	    update
	}

	if {$plot_test2 != ""} {
	    .canvasWin.canvasFrame.canvas itemconfigure $current_diag_group2 -fill $color_diagonal -outline $outline_diagonal
	    .canvasWin.canvasFrame.canvas raise $current_diag_group2
	    incr plot_counter1
	    puts "BA ::: $id_A ::: $id_B ::: $plot_counter1"

	    .canvasWin.canvasFrame.canvas itemconfigure linearPlotTagX$id_A -fill $color_diagonal -outline $outline_diagonal
	    .canvasWin.canvasFrame.canvas itemconfigure linearPlotTagX$id_B -fill $color_diagonal -outline $outline_diagonal
	    .canvasWin.canvasFrame.canvas raise linearPlotTagX$id_A
	    .canvasWin.canvasFrame.canvas raise linearPlotTagX$id_B

	    .canvasWin.canvasFrame.canvas itemconfigure linearPlotTagY$id_A -fill $color_diagonal -outline $outline_diagonal
	    .canvasWin.canvasFrame.canvas itemconfigure linearPlotTagY$id_B -fill $color_diagonal -outline $outline_diagonal
	    .canvasWin.canvasFrame.canvas raise linearPlotTagY$id_A
	    .canvasWin.canvasFrame.canvas raise linearPlotTagY$id_B
	    update
	}

	# update
	incr plot_counter2

    }
    update
    close $open_current_group
    set message_text "Diagonal group painted"
    .message_box config -text $message_text -bg blue4 -fg gray
    update
    bell
    puts "$plot_counter1 - ITEMS FOUND"
    puts "$plot_counter2 - TOTAL NUMBER OF ITEMS IN FILE"
}

# =====================================================

proc Canvas_Editor {} {

    destroy .c_editor
    toplevel .c_editor

    wm title .c_editor "Canvas Editor" 
    wm protocol .c_editor WM_DELETE_WINDOW "destroy .c_editor"
    wm minsize .c_editor 512 285

    global selected_canvas \
	font_color_canvas \
	font_color_label \
	canvas_editor_font_type \
	canvas_editor_font_size \
	canvas_editor_font_style \
	text_tag \
	canvas_editor_text \
	curX curY 

    global color_scheme

    if {$color_scheme == "WHITE"} {
	set font_color_canvas "black"
    }
    if {$color_scheme == "BLACK"} {
	set font_color_canvas "white"
    }

    set font_color_label $font_color_canvas
    set selected_canvas ".canvasWin.canvasFrame.canvas"
    set canvas_editor_text "my lovely gene"
    set canvas_editor_font_type "helvetica"
    set canvas_editor_font_size 12
    set canvas_editor_font_style normal
    set text_tag text_tag_01
    set curX 125
    set curY 125

    global graph_style \
	graph_tag \
	graphX1 graphX2 \
	graphY1 graphY2 \
	graph_fill_color \
	graph_outline_color

    set graph_style "rect"
    set graph_tag "graph_tag_01"
    set graphX1 100 
    set graphY1 100
    set graphX2 24
    set graphY2 12
    set graph_fill_color "cyan"
    set graph_outline_color "blue"

    label .c_editor.select_text_label -text "select text:" -bg yellow3 -width 12
    place .c_editor.select_text_label -x 10 -y 10

    entry .c_editor.select_text_entry -relief sunken \
	    -textvariable canvas_editor_text -bg gold -width 22
    place .c_editor.select_text_entry -x 105 -y 10

    label .c_editor.select_canvas -text "select canvas:" -bg gray -width 14
    place .c_editor.select_canvas -x 280 -y 10

    entry .c_editor.select_canvas_entry -relief sunken \
	    -textvariable selected_canvas -bg lightblue -width 15
    place .c_editor.select_canvas_entry -x 390 -y 10

    ########################

    label .c_editor.text_color_label -text "text color:" -bg gray -width 12
    place .c_editor.text_color_label -x 10 -y 40

    entry .c_editor.text_color_entry -relief sunken \
	    -textvariable font_color_label -bg lightyellow -width 20
    place .c_editor.text_color_entry -x 105 -y 40

    label .c_editor.select_graph_label -text "Graphical Label:" -bg yellow3 -width 18
    place .c_editor.select_graph_label -x 330 -y 40

    ########################

    label .c_editor.text_type_label -text "font type:" -bg gray -width 12
    place .c_editor.text_type_label -x 10 -y 70

    entry .c_editor.text_type_entry -relief sunken \
	    -textvariable canvas_editor_font_type -bg lightyellow -width 20
    place .c_editor.text_type_entry -x 105 -y 70

    label .c_editor.graph_style_label -text "shape:" -bg gray -width 12
    place .c_editor.graph_style_label -x 300 -y 70

    entry .c_editor.graph_style_entry -relief sunken \
	    -textvariable graph_style -bg lightyellow -width 14
    place .c_editor.graph_style_entry -x 395 -y 70

    ########################

    label .c_editor.text_size_label -text "font size:" -bg gray -width 12
    place .c_editor.text_size_label -x 10 -y 100

    entry .c_editor.text_size_entry -relief sunken \
	    -textvariable canvas_editor_font_size -bg lightyellow -width 20
    place .c_editor.text_size_entry -x 105 -y 100

    label .c_editor.graph_fill_label -text "fill:" -bg gray -width 12
    place .c_editor.graph_fill_label -x 300 -y 100

    entry .c_editor.graph_fill_entry -relief sunken \
	    -textvariable graph_fill_color -bg lightyellow -width 14
    place .c_editor.graph_fill_entry -x 395 -y 100

    ########################

    label .c_editor.text_style_label -text "font style:" -bg gray -width 12
    place .c_editor.text_style_label -x 10 -y 130

    entry .c_editor.text_style_entry -relief sunken \
	    -textvariable canvas_editor_font_style -bg lightyellow -width 20
    place .c_editor.text_style_entry -x 105 -y 130

    label .c_editor.graph_outline_label -text "outline:" -bg gray -width 12
    place .c_editor.graph_outline_label -x 300 -y 130

    entry .c_editor.graph_outline_entry -relief sunken \
	    -textvariable graph_outline_color -bg lightyellow -width 14
    place .c_editor.graph_outline_entry -x 395 -y 130

    ########################

    label .c_editor.current_X_label -text "current X:" -bg gray -width 12
    place .c_editor.current_X_label -x 10 -y 160

    label .c_editor.current_Y_label -text "current Y:" -bg gray -width 12
    place .c_editor.current_Y_label -x 10 -y 190

    entry .c_editor.current_X_entry -relief sunken \
	    -textvariable curX -bg lightyellow -width 20
    place .c_editor.current_X_entry -x 105 -y 160

    entry .c_editor.current_Y_entry -relief sunken \
	    -textvariable curY -bg lightyellow -width 20
    place .c_editor.current_Y_entry -x 105 -y 190

    label .c_editor.graphX1 -text "X1" -bg gray -width 4
    place .c_editor.graphX1 -x 300 -y 160

    label .c_editor.graphY1 -text "Y1" -bg gray -width 4
    place .c_editor.graphY1 -x 300 -y 190

    label .c_editor.graphX2 -text "X2" -bg gray -width 4
    place .c_editor.graphX2 -x 400 -y 160

    label .c_editor.graphY2 -text "Y2" -bg gray -width 4
    place .c_editor.graphY2 -x 400 -y 190

    entry .c_editor.graphX1_entry -relief sunken \
	    -textvariable graphX1 -bg lightyellow -width 6
    place .c_editor.graphX1_entry -x 340 -y 160

    entry .c_editor.graphY1_entry -relief sunken \
	    -textvariable graphY1 -bg lightyellow -width 6
    place .c_editor.graphY1_entry -x 340 -y 190

    entry .c_editor.graphX2_entry -relief sunken \
	    -textvariable graphX2 -bg lightyellow -width 6
    place .c_editor.graphX2_entry -x 440 -y 160

    entry .c_editor.graphY2_entry -relief sunken \
	    -textvariable graphY2 -bg lightyellow -width 6
    place .c_editor.graphY2_entry -x 440 -y 190

    ########################

    label .c_editor.tag_ID_label -text "active tag ID:" -bg yellow3 -width 12
    place .c_editor.tag_ID_label -x 10 -y 220

    entry .c_editor.tag_ID_entry -relief sunken \
	    -textvariable text_tag -bg gold -width 20
    place .c_editor.tag_ID_entry -x 105 -y 220

    label .c_editor.graph_ID_label -text "active tag ID:" -bg yellow3 -width 12
    place .c_editor.graph_ID_label -x 300 -y 220

    entry .c_editor.graph_ID_entry -relief sunken \
	    -textvariable graph_tag -bg gold -width 14
    place .c_editor.graph_ID_entry -x 395 -y 220

    ########################

    button .c_editor.submit_text -text "Add Text to Canvas" \
	    -command {Place_Text_on_Canvas} -bg orange -width 17
    place .c_editor.submit_text -x 20 -y 250

    button .c_editor.submit_graphics -text "Add Label to Canvas" \
	    -command {Add_Graphics_to_Canvas} -bg orange -width 17
    place .c_editor.submit_graphics -x 350 -y 250

    button .c_editor.how_to -text "HowTo" \
	    -command {How_To_Edit_Canvas} -bg gray -width 6
    place .c_editor.how_to -x 185 -y 250

    button .c_editor.dismiss -text "Dismiss" \
	    -command {destroy .c_editor} -bg gray -width 6
    place .c_editor.dismiss -x 265 -y 250
}

# =====================================================

proc Move_Text_Label { node xDist yDist } {

    global selected_canvas curX curY

    $selected_canvas move $node $xDist $yDist
    incr curX $xDist
    incr curY $yDist
}

# =====================================================

proc Move_Graph_Label { node xDist1 yDist1 } {

    global selected_canvas \
	graphX1 graphX2 \
	graphY1 graphY2

    $selected_canvas move $node $xDist1 $yDist1
    incr graphX1 $xDist1
    incr graphY1 $yDist1
    # incr graphX2 $xDist1
    # incr graphY2 $yDist1
}

# =====================================================

proc Place_Text_on_Canvas {} {

    global selected_canvas text_tag \
	curX curY canvas_editor_text \
	font_color_canvas font_color_label \
	canvas_editor_font_type \
	canvas_editor_font_size \
	canvas_editor_font_style

    $selected_canvas create text $curX $curY -text "$canvas_editor_text" \
	    -font "$canvas_editor_font_type $canvas_editor_font_size $canvas_editor_font_style" \
	    -anchor sw -fill $font_color_label -tag $text_tag

    $selected_canvas bind $text_tag <Button-1> {
	set curX [expr int(round([$selected_canvas canvasx %x]))]
	set curY [expr int(round([$selected_canvas canvasy %y]))]
    }
    $selected_canvas bind $text_tag <Shift-Button-1> {
	Delete_Text_Item
    }

    $selected_canvas bind $text_tag <B1-Motion> {
	Move_Text_Label [$selected_canvas find withtag current] \
	[expr [expr int(round([$selected_canvas canvasx %x]))] - $curX] \
	[expr [expr int(round([$selected_canvas canvasy %y]))] - $curY]
    set curX [expr int(round([$selected_canvas canvasx %x]))]
    set curY [expr int(round([$selected_canvas canvasy %y]))]
    }
}

# =====================================================

proc Add_Graphics_to_Canvas {} {

    global  graph_style \
	    graph_tag \
	    graphX1 graphX2 \
	    graphY1 graphY2 \
	    graph_fill_color \
	    graph_outline_color \
	    selected_canvas

    $selected_canvas create $graph_style $graphX1 $graphY1 \
	[expr $graphX1 + $graphX2] \
	[expr $graphY1 + $graphY2] \
	-fill $graph_fill_color -outline $graph_outline_color -tag $graph_tag

    $selected_canvas bind $graph_tag <Button-1> {
    set graphX1 [expr int(round([$selected_canvas canvasx %x]))]
    set graphY1 [expr int(round([$selected_canvas canvasy %y]))]
    }

    $selected_canvas bind $graph_tag <Shift-Button-1> {
	Delete_Graph_Item
    }

    $selected_canvas bind $graph_tag <B1-Motion> {
	Move_Graph_Label [$selected_canvas find withtag current] \
	[expr [expr int(round([$selected_canvas canvasx %x]))] - $graphX1] \
	[expr [expr int(round([$selected_canvas canvasy %y]))] - $graphY1] 
    set graphX1 [expr int(round([$selected_canvas canvasx %x]))]
    set graphY1 [expr int(round([$selected_canvas canvasy %y]))]
    }
}

# =====================================================

# DELETE TEXT ITEM
proc Delete_Text_Item {} {

    global selected_canvas text_tag

    toplevel .delete_item
    wm title .delete_item "WARNING!" 
    wm protocol .delete_item WM_DELETE_WINDOW "destroy .delete_item"

    label .delete_item.text -text "Do you really want to delete 
all labels with tag ID: 

$text_tag\?"

    pack .delete_item.text -padx 7 -pady 7

    button .delete_item.yes -text "Yes" -command {Destroy_Text_and_Dismiss} \
	    -bg orange -width 4
    button .delete_item.no -text "No" -command "destroy .delete_item" \
	    -bg lightblue -width 4

    pack .delete_item.yes -padx 3 -pady 3
    pack .delete_item.no -padx 3 -pady 3
}

proc Destroy_Text_and_Dismiss {} {
    global selected_canvas text_tag
    $selected_canvas delete $text_tag
    destroy .delete_item
}

# =====================================================

# DELETE GRAPH ITEM
proc Delete_Graph_Item {} {

    global selected_canvas graph_tag

    toplevel .delete_graph_item
    wm title .delete_graph_item "WARNING!" 
    wm protocol .delete_graph_item WM_DELETE_WINDOW "destroy .delete_graph_item"

    label .delete_graph_item.text -text "Do you really want to delete 
all labels with tag ID: 

$graph_tag\?"

    pack .delete_graph_item.text -padx 7 -pady 7

    button .delete_graph_item.yes -text "Yes" -command {Destroy_Graph_and_Dismiss} \
	    -bg orange -width 4
    button .delete_graph_item.no -text "No" -command "destroy .delete_graph_item" \
	    -bg lightblue -width 4

    pack .delete_graph_item.yes -padx 3 -pady 3
    pack .delete_graph_item.no -padx 3 -pady 3
}

# =====================================================

proc Destroy_Graph_and_Dismiss {} {
    global selected_canvas graph_tag
    $selected_canvas delete $graph_tag
    destroy .delete_graph_item
}

# =====================================================

proc Print_Gene_ID {gene_id1 gene_id2} {

    global selected_canvas text_tag \
	curX curY canvas_editor_text \
	font_color_canvas font_color_label \
	canvas_editor_font_type \
	canvas_editor_font_size \
	canvas_editor_font_style

    if { $gene_id2 == "just_one" } {
	set gene_id2print ""
    } else {
	set gene_id2print $gene_id2
    }

    set is_it_here [winfo exists .c_editor]
    if {$is_it_here == 0} {
	Canvas_Editor
    }

    $selected_canvas bind $text_tag <Enter> {
	set curX [expr int(round([$selected_canvas canvasx %x]))]
	set curY [expr int(round([$selected_canvas canvasy %y]))]
    }

    $selected_canvas create text [expr $curX ] [expr $curY + 10] -text "$gene_id1\n$gene_id2print" \
	    -font "$canvas_editor_font_type $canvas_editor_font_size $canvas_editor_font_style" \
	    -anchor sw -fill $font_color_label -tag $text_tag

    $selected_canvas bind $text_tag <Button-1> {
	set curX [expr int(round([$selected_canvas canvasx %x]))]
	set curY [expr int(round([$selected_canvas canvasy %y]))]
    }
    $selected_canvas bind $text_tag <Shift-Button-1> {
	Delete_Text_Item
    }

    $selected_canvas bind $text_tag <B1-Motion> {
	Move_Text_Label [$selected_canvas find withtag current] \
	[expr [expr int(round([$selected_canvas canvasx %x]))] - $curX] \
	[expr [expr int(round([$selected_canvas canvasy %y]))] - $curY]
    set curX [expr int(round([$selected_canvas canvasx %x]))]
    set curY [expr int(round([$selected_canvas canvasy %y]))]
    }     
}

# =====================================================

proc How_To_Edit_Canvas {} {

    destroy .how_to_edit_canvas
    toplevel .how_to_edit_canvas
    wm title .how_to_edit_canvas "Canvas Editing HowTo" 
    wm protocol .how_to_edit_canvas WM_DELETE_WINDOW "destroy .how_to_edit_canvas"

    label .how_to_edit_canvas.text -text "
By using this Canvas Editor you may edit and finish your 
project(s) by adding text and graphical labels over the 
image generated by GenomePixelizer 2D plot. 
By left-button mouse click you may move the text and 
labels over GenomePixelizer canvas.
By holding <Shift> and left-button mouse click you may 
delete the text and labels you don't need. 

"
    button .how_to_edit_canvas.dismiss -text "Dismiss" \
	-command "destroy .how_to_edit_canvas" -bg gray -width 8

    pack .how_to_edit_canvas.text -padx 7 -pady 7
    pack .how_to_edit_canvas.dismiss -padx 3 -pady 3
}

# =====================================================

proc HowTo_PhyloWebify {} {

    destroy .fly
    set t [Scrolled_Text .fly -width 80 -height 40]
    set in [open "genoPix2D.README" r]
    $t insert end [read $in]
    close $in
    wm title .fly "GenomePixelizer 2D plotter"
    wm protocol .fly WM_DELETE_WINDOW "destroy .fly"
}

# =====================================================

proc Scrolled_Text { fly args } {

    toplevel $fly
    eval {text $fly.text -wrap none \
           -xscrollcommand [list $fly.xscroll set] \
           -yscrollcommand [list $fly.yscroll set]} $args
    scrollbar $fly.xscroll -orient horizontal \
              -command [list $fly.text xview]
    scrollbar $fly.yscroll -orient vertical \
              -command [list $fly.text yview]
    grid $fly.text $fly.yscroll -sticky news
    grid $fly.xscroll -sticky news
    grid rowconfigure $fly 0 -weight 1
    grid columnconfigure $fly 0 -weight 1
    button $fly.dismiss -text "Dismiss" -command "destroy $fly"
    grid $fly.dismiss
    
    return $fly.text
}

# =====================================================

#######################################################
#               MAIN FUNCTION                         #
#######################################################

wm title . "GenomePixelizer 2D Plotter"
wm protocol . WM_DELETE_WINDOW "destroy	."
# wm minsize . 580 320
. config -bg "#000055"

displayGUI

#######################################################
#                   THE END                           #
#######################################################
